#!/bin/sh
#
##
# clpgcdns-stop.sh
##
# stop the clpgcdns
##


echo "clpgcdns stop script start."


##
# success & error code
##
GCDNS_SUCCESS=0								# succeed
GCDNS_ERR_GET_VALUE=1						# failed to get the setting value
GCDNS_ERR_GET_RECORD=2						# get dns record has failed
GCDNS_ERR_PARAM=3							# invalid parameter
GCDNS_ERR_TRN_START=4						# start transaction has failed
GCDNS_ERR_REMOVE_RECORD=5					# remove dns record has failed
GCDNS_ERR_TRN_EXEC=6						# transaction execute has failed


##
# set value
##
ZONE_NAME="${CLP_OCF_PARAM1}"				# zone name (ex. "test-zone")
RECORD_NAME="${CLP_OCF_PARAM2}"				# dns record (ex. "test-cluster.example.com")
RECORD_TYPE="${CLP_OCF_PARAM3}"				# record type (hidden settings) (ex. "A")
RECORD_DELETE_FLAG="${CLP_OCF_PARAM6}"		# delete records when inactive flag

RETRY=10									# number of transaction execution result confirmations
INTERVAL=1									# transaction execution result confirmation interval

LOGCMD_ID=3601								# event id of clplogcmd

if [ -z "$ZONE_NAME" ] || [ -z "$RECORD_NAME" ] || [ -z "$RECORD_TYPE" ] || [ -z "$RECORD_DELETE_FLAG" ]
then
	clplogcmd -m "[${GCDNS_ERR_GET_VALUE}] gcdns: Failed to obtain the setting value in the deactivation process." --alert --syslog -l ERR -i ${LOGCMD_ID} > /dev/null 2>&1
	echo "failed to get the setting value." 1>&2
	echo "clpgcdns stop script has failed." 1>&2
	exit ${GCDNS_ERR_GET_VALUE}
fi

##
# get dns record
##
CURRENT="`gcloud dns record-sets list -z=\"${ZONE_NAME}\" --name=\"${RECORD_NAME}\" --type=\"${RECORD_TYPE}\" \
	--format=\"table[no-heading](name,type,ttl, rrdatas.list():label=DATA)\"`"
if [ $? -ne 0 ]
then
	clplogcmd -m "[${GCDNS_ERR_GET_RECORD}] gcdns: Failed to obtain the record set in the deactivation process." --alert --syslog -l ERR -i ${LOGCMD_ID} > /dev/null 2>&1
	echo "get dns record has failed." 1>&2
	echo "clpgcdns stop script has failed." 1>&2
	exit ${GCDNS_ERR_GET_RECORD}
elif [ -z "$CURRENT" ]
then
	echo "there is no designated dns record."
	echo "clpgcdns stop script has succeeded."
	exit ${GCDNS_SUCCESS}
else
	echo "get dns record has succeeded."
	
	CURRENT_NAME="`echo ${CURRENT} | awk '{print $1}'`"
	CURRENT_TYPE="`echo ${CURRENT} | awk '{print $2}'`"
	CURRENT_TTL="`echo ${CURRENT} | awk '{print $3}'`"
	CURRENT_IP="`echo ${CURRENT} | awk '{print $4}'`"
fi



##
# delete records when inactive flag
##
if [ $RECORD_DELETE_FLAG -eq 0 ]
then
	echo "clpgcdns stop script has succeeded."
	exit ${GCDNS_SUCCESS}
elif [ $RECORD_DELETE_FLAG -eq 1 ]
then
	: #do nothing
else
	clplogcmd -m "[${GCDNS_ERR_PARAM}] gcdns: Detected an invalid parameter in the deactivation process." --alert --syslog -l ERR -i ${LOGCMD_ID} > /dev/null 2>&1
	echo "invalid parameter." 1>&2
	echo "clpgcdns stop script has failed." 1>&2
	exit ${GCDNS_ERR_PARAM}
fi


##
# abort transaction
##
gcloud dns record-sets transaction abort -z="${ZONE_NAME}" > /dev/null 2>&1


##
#
# unegister dns record
#
##


##
# start transaction
##
gcloud dns record-sets transaction start -z="${ZONE_NAME}"
if [ $? -ne 0 ]
then
	gcloud dns record-sets transaction abort -z="${ZONE_NAME}" > /dev/null 2>&1
	clplogcmd -m "[${GCDNS_ERR_TRN_START}] gcdns: Failed to start the transaction in the deactivation process." --alert --syslog -l ERR -i ${LOGCMD_ID} > /dev/null 2>&1
	echo "start transaction has failed." 1>&2
	echo "clpgcdns stop script has failed." 1>&2
	exit ${GCDNS_ERR_TRN_START}
fi


##
# remove dns record
##
gcloud dns record-sets transaction remove -z="${ZONE_NAME}" --name="${CURRENT_NAME}" \
	--ttl="${CURRENT_TTL}" --type="${CURRENT_TYPE}" "${CURRENT_IP}"
if [ $? -ne 0 ]
then
	gcloud dns record-sets transaction abort -z="${ZONE_NAME}" > /dev/null 2>&1
	clplogcmd -m "[${GCDNS_ERR_REMOVE_RECORD}] gcdns: Failed to delete the record set in the deactivation process." --alert --syslog -l ERR -i ${LOGCMD_ID} > /dev/null 2>&1
	echo "remove dns record has failed." 1>&2
	echo "clpgcdns stop script has failed." 1>&2
	exit ${GCDNS_ERR_REMOVE_RECORD}
fi


##
# execute transaction
##
CHANGE_ID="`gcloud dns record-sets transaction execute -z=\"${ZONE_NAME}\" --format=\"table[no-heading](id)\"`"
if [ $? -ne 0 ]
then
	gcloud dns record-sets transaction abort -z="${ZONE_NAME}" > /dev/null 2>&1
	clplogcmd -m "[${GCDNS_ERR_TRN_EXEC}] gcdns: Failed to execute the transaction in the deactivation process." --alert --syslog -l ERR -i ${LOGCMD_ID} > /dev/null 2>&1
	echo "transaction execute has failed." 1>&2
	echo "clpgcdns stop script has failed." 1>&2
	exit ${GCDNS_ERR_TRN_EXEC}
fi


##
# confirmation of transaction execution result
##
for i in `seq ${RETRY}`
do
	sleep ${INTERVAL}
	
	STATUS="`gcloud dns record-sets changes describe \"${CHANGE_ID}\" -z=\"${ZONE_NAME}\" --format=\"table[no-heading](status)\"`"
	
	if [ $? -ne 0 ]
	then
		echo "check the transaction execution result has failed." 1>&2
		echo "clpgcdns stop script has succeeded."
		exit ${GCDNS_SUCCESS}
	elif [ "$STATUS" = "done" ]
	then
		echo "check the transaction execution result has succeeded."
		break
	fi
done

if [ "$STATUS" != "done" ]
then
	echo "transaction execution status is ${STATUS}." 1>&2
fi


##
# end
##
echo "clpgcdns stop script has succeeded."

exit ${GCDNS_SUCCESS}

